@@ -2,7 +2,9 @@
 @relation(SDOC-SRS-98, SDOC-SRS-109, scope=file)
 """
 
-from typing import Generator, List, Optional
+from collections import defaultdict
+from dataclasses import dataclass
+from typing import DefaultDict, Dict, Generator, List, Optional, Set
 
 from strictdoc.backend.sdoc.document_reference import DocumentReference
 from strictdoc.backend.sdoc.models.document_config import DocumentConfig
@@ -23,12 +25,13 @@ from strictdoc.backend.sdoc.models.model import (
     SDocElementIF,
     SDocNodeIF,
 )
-from strictdoc.backend.sdoc.models.node import SDocNode
+from strictdoc.backend.sdoc.models.node import SDocNode, SDocNodeField
 from strictdoc.core.document_meta import DocumentMeta
 from strictdoc.helpers.auto_described import auto_described
 from strictdoc.helpers.cast import assert_cast
 from strictdoc.helpers.mid import MID
 from strictdoc.helpers.ordered_set import OrderedSet
+from strictdoc.helpers.string import tokenize
 
 
 @auto_described
@@ -37,6 +40,18 @@ class SDocDocumentContext:
         self.title_number_string: Optional[str] = None
 
 
+@dataclass
+class SDocDocumentSearchIndex:
+    document_index: DefaultDict[str, Set[str]]
+    map_nodes_by_mid: Dict[str, Dict[str, str]]
+
+    @classmethod
+    def create_empty(cls) -> "SDocDocumentSearchIndex":
+        return SDocDocumentSearchIndex(
+            document_index=defaultdict(set), map_nodes_by_mid={}
+        )
+
+
 @auto_described
 class SDocDocument(SDocDocumentIF):
     def __init__(
@@ -48,6 +63,7 @@ class SDocDocument(SDocDocumentIF):
         grammar: Optional[DocumentGrammar],
         section_contents: List[SDocElementIF],
         is_bundle_document: bool = False,
+        autogen: bool = False,
     ) -> None:
         self.title: str = title
         self.reserved_title: str = title
@@ -66,6 +82,7 @@ class SDocDocument(SDocDocumentIF):
 
         self.fragments_from_files: List[SDocDocumentFromFileIF] = []
 
+        # FIXME: Is this used?
         self.ng_level: int = 0
         self.ng_has_requirements = False
 
@@ -81,6 +98,24 @@ class SDocDocument(SDocDocumentIF):
             SDocDocumentFromFileIF
         ] = None
 
+        self.ng_whitelisted: bool = True
+        self.search_index = SDocDocumentSearchIndex.create_empty()
+
+        # Specifies whether a node is created from text or autogenerated, e.g.,
+        # from a JUnit XML test report or from reading source file comments.
+        # The SDoc writer uses this property to decide whether it shall write
+        # autogenerated code to disk.
+        self.autogen: bool = autogen
+
+    def get_total_size(self) -> int:
+        if self.section_contents is None or len(self.section_contents) == 0:
+            return 0
+        total_size = 0
+        for node_ in self.section_contents:
+            if isinstance(node_, SDocNode):
+                total_size += node_.get_total_size()
+        return total_size
+
     def iterate_nodes(
         self, element_type: Optional[str] = None
     ) -> Generator[SDocNodeIF, None, None]:
@@ -174,7 +209,7 @@ class SDocDocument(SDocDocumentIF):
 
     def iterate_included_documents_depth_first(
         self,
-    ) -> Generator["SDocDocument", None, None]:
+    ) -> Generator["SDocDocumentIF", None, None]:
         for included_document_ in self.included_documents:
             yield included_document_
             yield from included_document_.iterate_included_documents_depth_first()
@@ -265,3 +300,68 @@ class SDocDocument(SDocDocumentIF):
             return self.collect_options_for_tag(element_type, field_name)
 
         raise AssertionError(f"Must not reach here: {field}")
+
+    def blacklist_if_needed(self) -> None:
+        if self.section_contents is not None:
+            for node_ in self.section_contents:
+                if node_.ng_whitelisted:
+                    return
+
+        self.ng_whitelisted = False
+
+    def build_search_index(self) -> None:
+        """
+        Build a static search index for this document.
+
+        @relation(SDOC-SRS-155, scope=function)
+        """
+
+        document_index = defaultdict(set)
+        map_nodes_by_mid = {}
+
+        from strictdoc.core.document_iterator import (  # noqa: PLC0415
+            DocumentCachingIterator,
+        )
+
+        document_iterator = DocumentCachingIterator(self)
+
+        for node, _ in document_iterator.all_content(
+            print_fragments=False,
+        ):
+            if not isinstance(node, SDocNode):
+                continue
+
+            node_dict = {}
+
+            node_dict["MID"] = node.reserved_mid.get_string_value()
+            map_nodes_by_mid[node.reserved_mid.get_string_value()] = node_dict
+
+            for (
+                field_name_,
+                field_values_,
+            ) in node.ordered_fields_lookup.items():
+                requirement_field: SDocNodeField = field_values_[0]
+                requirement_field_value = requirement_field.get_text_value()
+
+                node_dict[field_name_] = requirement_field_value
+
+                tokens = set(tokenize(requirement_field_value))
+                for token in tokens:
+                    if len(token) > 1:
+                        document_index[token].add(
+                            node.reserved_mid.get_string_value()
+                        )
+
+                        for i in range(0, len(token)):
+                            token_incremental = token[: i + 1]
+                            document_index[token_incremental].add(
+                                node.reserved_mid
+                            )
+                            token_deincremental = token[i:]
+                            document_index[token_deincremental].add(
+                                node.reserved_mid
+                            )
+
+        self.search_index = SDocDocumentSearchIndex(
+            document_index, map_nodes_by_mid
+        )
