@@ -51,7 +51,7 @@ class SDocNodeField:
         multiline__: Optional[str],
     ) -> None:
         self.parent: Optional[SDocNode] = parent
-        self.field_name = field_name
+        self.field_name: str = field_name
         self.parts: List[Any] = parts
         self.multiline: bool = multiline__ is not None and len(multiline__) > 0
 
@@ -110,6 +110,7 @@ class SDocNode(SDocNodeIF):
         is_composite: bool = False,
         section_contents: Optional[List[SDocElementIF]] = None,
         node_type_close: Optional[str] = None,
+        autogen: bool = False,
     ) -> None:
         assert parent
         assert isinstance(node_type, str)
@@ -193,6 +194,43 @@ class SDocNode(SDocNodeIF):
 
         self.ng_has_requirements: bool = False
 
+        # Specifies whether a node is created from text or autogenerated, e.g.,
+        # from a JUnit XML test report or from reading source file comments.
+        # The SDoc writer uses this property to decide whether it shall write
+        # autogenerated code to disk.
+        self.autogen: bool = autogen
+
+    def get_total_size(self) -> int:
+        if self.section_contents is None or len(self.section_contents) == 0:
+            return 1
+        total_size = 0
+        for node_ in self.section_contents:
+            if isinstance(node_, SDocNode):
+                total_size += node_.get_total_size()
+        return total_size
+
+    @staticmethod
+    def create_section(
+        parent: Any, document: SDocDocumentIF, title: str
+    ) -> "SDocCompositeNode":
+        node = SDocCompositeNode(
+            parent=parent,
+            node_type="SECTION",
+            fields=[],
+            relations=[],
+            section_contents=[],
+            node_type_close="SECTION",
+        )
+        node.ng_including_document_reference = DocumentReference()
+        node.ng_document_reference = DocumentReference()
+        node.ng_document_reference.set_document(document)
+        node.set_field_value(
+            field_name="TITLE",
+            form_field_index=0,
+            value=title,
+        )
+        return node
+
     @staticmethod
     def get_type_string() -> str:
         return "requirement"
@@ -263,6 +301,14 @@ class SDocNode(SDocNodeIF):
             RequirementFieldName.UID, singleline_only=True
         )
 
+    @reserved_uid.setter
+    def reserved_uid(self, uid: Optional[str]) -> None:
+        self.set_field_value(
+            field_name="UID",
+            form_field_index=0,
+            value=uid,
+        )
+
     @property
     def reserved_status(self) -> Optional[str]:
         return self._get_cached_field(
@@ -313,7 +359,7 @@ class SDocNode(SDocNodeIF):
         return True
 
     def is_normative_node(self) -> bool:
-        return not self.is_text_node()
+        return self.node_type not in ("SECTION", "TEXT")
 
     def is_text_node(self) -> bool:
         return self.node_type == "TEXT"
@@ -366,11 +412,6 @@ class SDocNode(SDocNodeIF):
             debug_components.append(f"document = {document.get_debug_info()}")
         return f"Requirement({', '.join(debug_components)})"
 
-    # FIXME: Remove this method. Use get_parent_or_including_document() instead.
-    @property
-    def parent_or_including_document(self) -> SDocDocumentIF:
-        return self.get_parent_or_including_document()
-
     def document_is_included(self) -> bool:
         assert self.ng_including_document_reference is not None
         return self.ng_including_document_reference.get_document() is not None
@@ -593,6 +634,19 @@ class SDocNode(SDocNodeIF):
             )
         )
 
+    def blacklist_if_needed(self) -> None:
+        if self.section_contents is not None:
+            for node_ in self.section_contents:
+                if node_.ng_whitelisted:
+                    return
+
+        self.ng_whitelisted = False
+
+        # If it turns out that all child nodes are blacklisted,
+        # go up and blacklist the parent node if needed.
+        if self.parent.ng_whitelisted:
+            self.parent.blacklist_if_needed()
+
     def _get_cached_field(
         self, field_name: str, singleline_only: bool
     ) -> Optional[str]:
