@@ -1,6 +1,5 @@
-# mypy: disable-error-code="arg-type,no-untyped-call,union-attr"
 import os.path
-from typing import Callable, Dict, List, Optional, cast
+from typing import Callable, Dict, List, Optional, Union, cast
 
 from textx import TextXSyntaxError, get_model
 
@@ -11,7 +10,12 @@ from strictdoc.backend.sdoc.models.document_from_file import DocumentFromFile
 from strictdoc.backend.sdoc.models.document_grammar import DocumentGrammar
 from strictdoc.backend.sdoc.models.document_view import DocumentView
 from strictdoc.backend.sdoc.models.grammar_element import GrammarElement
-from strictdoc.backend.sdoc.models.model import SDocDocumentFromFileIF
+from strictdoc.backend.sdoc.models.model import (
+    SDocDocumentFromFileIF,
+    SDocDocumentIF,
+    SDocNodeIF,
+    SDocSectionIF,
+)
 from strictdoc.backend.sdoc.models.node import (
     SDocNode,
     SDocNodeField,
@@ -53,7 +57,7 @@ class SDocParsingProcessor:
             or DocumentGrammar.create_default(
                 document,
                 create_section_element=self.parse_context.migrate_sections,
-                enable_mid=document.config.enable_mid,
+                enable_mid=document.config.enable_mid or False,
             )
         )
         document.ng_including_document_reference = (
@@ -86,11 +90,17 @@ class SDocParsingProcessor:
     def process_document_grammar(
         self, document_grammar: DocumentGrammar
     ) -> None:
+        assert self.parse_context.document_config is not None
+
         preserve_source_location_data(document_grammar)
         # FIXME: It would be great to move forward and remove this.
         if not document_grammar.has_text_element():
             document_grammar.add_element_first(
-                DocumentGrammar.create_default_text_element(document_grammar)
+                DocumentGrammar.create_default_text_element(
+                    document_grammar,
+                    enable_mid=self.parse_context.document_config.enable_mid
+                    is True,
+                )
             )
         self.parse_context.document_grammar = document_grammar
 
@@ -115,6 +125,10 @@ class SDocParsingProcessor:
             self.parse_context.context_document_reference
         )
         preserve_source_location_data(section)
+
+        # FIXME: Refactor to eliminate the need in such assert.
+        assert self.parse_context.document_config is not None
+
         if self.parse_context.document_config.auto_levels:
             if (
                 section.ng_resolved_custom_level
@@ -143,6 +157,7 @@ class SDocParsingProcessor:
         )
 
         # Windows paths are backslashes, so using abspath in addition.
+        assert self.parse_context.path_to_sdoc_dir is not None
         resolved_path_to_fragment_file = os.path.abspath(
             os.path.join(
                 self.parse_context.path_to_sdoc_dir, document_from_file.file
@@ -169,8 +184,6 @@ class SDocParsingProcessor:
         self.parse_context.fragments_from_files.append(document_from_file)
 
     def process_requirement(self, requirement: SDocNode) -> None:
-        self.parse_context.document_has_requirements = True
-
         requirement.ng_document_reference = (
             self.parse_context.document_reference
         )
@@ -178,14 +191,20 @@ class SDocParsingProcessor:
             self.parse_context.context_document_reference
         )
 
-        cursor = requirement.parent
-        while (
-            not isinstance(cursor, SDocDocument)
-            and not cursor.ng_has_requirements
-        ):
-            cursor.ng_has_requirements = True
-            cursor = cursor.parent
+        if requirement.is_normative_node():
+            self.parse_context.document_has_requirements = True
 
+            cursor: Union[SDocDocumentIF, SDocSectionIF, SDocNodeIF] = (
+                requirement.parent
+            )
+            while (
+                isinstance(cursor, (SDocSectionIF, SDocNodeIF))
+                and not cursor.ng_has_requirements
+            ):
+                cursor.ng_has_requirements = True
+                cursor = cursor.parent
+
+        assert self.parse_context.document_config is not None
         if (
             requirement.reserved_title is None
             or not self.parse_context.document_config.is_requirement_in_toc()
