@@ -5,7 +5,9 @@ from textx import TextXSemanticError, TextXSyntaxError, metamodel_from_str
 
 from strictdoc.backend.sdoc.grammar.grammar_builder import SDocGrammarBuilder
 from strictdoc.backend.sdoc.models.constants import DOCUMENT_MODELS
-from strictdoc.backend.sdoc.models.document import SDocDocument
+from strictdoc.backend.sdoc.models.document import (
+    SDocDocument,
+)
 from strictdoc.backend.sdoc.models.document_grammar import DocumentGrammar
 from strictdoc.backend.sdoc.models.node import (
     SDocCompositeNode,
@@ -18,6 +20,8 @@ from strictdoc.backend.sdoc.processor import ParseContext, SDocParsingProcessor
 from strictdoc.core.project_config import ProjectConfig
 from strictdoc.helpers.cast import assert_cast
 from strictdoc.helpers.exception import StrictDocException
+from strictdoc.helpers.file_system import file_open_read_utf8
+from strictdoc.helpers.string import strip_bom
 from strictdoc.helpers.textx import drop_textx_meta
 
 
@@ -34,6 +38,8 @@ class SDReader:
         file_path: Optional[str] = None,
         migrate_sections: bool = False,
     ) -> Tuple[SDocDocument, ParseContext]:
+        input_string = strip_bom(input_string)
+
         parse_context = ParseContext(
             path_to_sdoc_file=file_path, migrate_sections=migrate_sections
         )
@@ -98,7 +104,7 @@ class SDReader:
         if unpickled_content:
             return assert_cast(unpickled_content, SDocDocument)
 
-        with open(file_path, encoding="utf8") as file:
+        with file_open_read_utf8(file_path) as file:
             sdoc_content = file.read()
 
         sdoc, parse_context = self.read_with_parse_context(
@@ -115,6 +121,10 @@ class SDReader:
         # are not used anyway.
         drop_textx_meta(sdoc)
 
+        # @relation(SDOC-SRS-155, scope=range_start)
+        sdoc.build_search_index()
+        # @relation(SDOC-SRS-155, scope=range_end)
+
         PickleCache.save_to_cache(sdoc, file_path, project_config, "sdoc")
 
         return sdoc
