@@ -1,7 +1,6 @@
-# mypy: disable-error-code="arg-type,no-untyped-call,union-attr"
 import os.path
 from pathlib import Path
-from typing import Dict, List, Tuple, Union
+from typing import Dict, List, Tuple
 
 from strictdoc.backend.sdoc.models.document import SDocDocument
 from strictdoc.backend.sdoc.models.document_config import DocumentConfig
@@ -20,6 +19,9 @@ from strictdoc.backend.sdoc.models.grammar_element import (
     GrammarElementRelationType,
     RequirementFieldType,
 )
+from strictdoc.backend.sdoc.models.model import (
+    SDocElementIF,
+)
 from strictdoc.backend.sdoc.models.node import (
     SDocCompositeNode,
     SDocNode,
@@ -292,10 +294,15 @@ class SDWriter:
 
     def _print_node(
         self,
-        root_node: Union[SDocDocument, SDocSection, SDocNode, DocumentFromFile],
+        root_node: SDocElementIF,
         document: SDocDocument,
         document_iterator: DocumentCachingIterator,
     ) -> str:
+        # Currently, auto-generated nodes are never written back to file system.
+        # We could revisit this in the future.
+        if root_node.autogen:
+            return ""
+
         assert isinstance(document_iterator, DocumentCachingIterator), (
             document_iterator
         )
@@ -400,9 +407,9 @@ class SDWriter:
             output += section.reserved_mid
             output += "\n"
 
-        if section.uid:
+        if section.reserved_uid:
             output += "UID: "
-            output += section.uid
+            output += section.reserved_uid
             output += "\n"
 
         if section.custom_level:
@@ -435,6 +442,8 @@ class SDWriter:
     def _print_requirement_fields(
         self, section_content: SDocNode, document: SDocDocument
     ) -> str:
+        assert document.grammar is not None
+
         output = ""
 
         current_view: ViewElement = document.view.get_current_view(
