@@ -4,7 +4,7 @@
 
 import os
 from enum import IntEnum
-from typing import Any, List, Optional, Union
+from typing import Any, List, Optional, Type, Union
 
 import bs4
 from bs4 import BeautifulSoup
@@ -18,10 +18,10 @@ from strictdoc.backend.sdoc.models.reference import (
     FileEntryFormat,
     FileReference,
 )
-from strictdoc.backend.sdoc.models.section import SDocSection
 from strictdoc.core.file_tree import File
 from strictdoc.core.project_config import ProjectConfig
 from strictdoc.helpers.cast import assert_cast, assert_optional_cast
+from strictdoc.helpers.file_system import file_open_read_utf8
 from strictdoc.helpers.paths import path_to_posix_path
 
 
@@ -47,15 +47,17 @@ class JUnitXMLFormat(IntEnum):
 class JUnitXMLReader:
     @classmethod
     def read_from_file(
-        cls: "JUnitXMLReader", doc_file: File, project_config: ProjectConfig
+        cls: Type["JUnitXMLReader"],
+        doc_file: File,
+        project_config: ProjectConfig,
     ) -> SDocDocument:
-        with open(doc_file.full_path, encoding="UTF-8") as file:
+        with file_open_read_utf8(doc_file.full_path) as file:
             content = file.read()
         return cls.read_from_string(content, doc_file, project_config)
 
     @classmethod
     def read_from_string(
-        cls: "JUnitXMLReader",
+        cls: Type["JUnitXMLReader"],
         content: str,
         doc_file: File,
         project_config: ProjectConfig,
@@ -79,6 +81,7 @@ class JUnitXMLReader:
             view=None,
             grammar=None,
             section_contents=[],
+            autogen=True,
         )
         document.ng_including_document_reference = DocumentReference()
         grammar = DocumentGrammar.create_for_test_report(document)
@@ -123,20 +126,11 @@ class JUnitXMLReader:
             total_skipped: int = int(str(xml_testsuite_["skipped"]))
             total_success: int = total_tests - total_failures - total_skipped
 
-            test_suite_section = SDocSection(
+            test_suite_section = SDocNode.create_section(
                 parent=document,
-                mid=None,
-                uid=None,
-                custom_level=None,
+                document=document,
                 title=title,
-                requirement_prefix=None,
-                section_contents=[],
             )
-            test_suite_section.ng_including_document_reference = (
-                DocumentReference()
-            )
-            test_suite_section.ng_document_reference = DocumentReference()
-            test_suite_section.ng_document_reference.set_document(document)
             document.section_contents.append(test_suite_section)
 
             summary_table = f"""\
