@@ -1,3 +1,7 @@
+"""
+@relation(SDOC-SRS-104, scope=file)
+"""
+
 import os
 import sys
 from functools import partial
@@ -88,9 +92,7 @@ class DocumentFinder:
                 assert isinstance(document_or_grammar, DocumentGrammar)
             elif doc_full_path.endswith(".reqif"):
                 reqif_reader = ReqIFReader()
-                reqif_documents = reqif_reader.read_from_file(
-                    doc_full_path, project_config
-                )
+                reqif_documents = reqif_reader.read_from_file(doc_full_path)
                 assert len(reqif_documents) >= 0
                 document_or_grammar = reqif_documents[0]
                 assert isinstance(document_or_grammar, SDocDocument)
