@@ -1,4 +1,3 @@
-# mypy: disable-error-code="arg-type,no-untyped-call"
 import datetime
 import os
 import re
@@ -48,6 +47,7 @@ class ProjectFeature(str, Enum):
     REQIF = "REQIF"
     DIFF = "DIFF"
     PROJECT_STATISTICS_SCREEN = "PROJECT_STATISTICS_SCREEN"
+    TREE_MAP_SCREEN = "TREE_MAP_SCREEN"
     STANDALONE_DOCUMENT_SCREEN = "STANDALONE_DOCUMENT_SCREEN"
     TRACEABILITY_MATRIX_SCREEN = "TRACEABILITY_MATRIX_SCREEN"
     REQUIREMENT_TO_SOURCE_TRACEABILITY = "REQUIREMENT_TO_SOURCE_TRACEABILITY"
@@ -98,6 +98,7 @@ class ProjectConfig:
         exclude_source_paths: List[str],
         test_report_root_dict: Dict[str, str],
         source_nodes: List[Dict[str, str]],
+        html2pdf_strict: bool,
         html2pdf_template: Optional[str],
         bundle_document_version: Optional[str],
         bundle_document_date: Optional[str],
@@ -111,6 +112,7 @@ class ProjectConfig:
         config_last_update: Optional[datetime.datetime],
         chromedriver: Optional[str],
         section_behavior: Optional[str],
+        statistics_generator: Optional[str],
     ) -> None:
         assert isinstance(environment, SDocRuntimeEnvironment)
         if source_root_path is not None:
@@ -172,6 +174,7 @@ class ProjectConfig:
 
         self.excel_export_fields: Optional[List[str]] = None
 
+        self.html2pdf_strict: bool = html2pdf_strict
         self.html2pdf_template: Optional[str] = html2pdf_template
         self.bundle_document_version: Optional[str] = bundle_document_version
         self.bundle_document_date: Optional[str] = bundle_document_date
@@ -206,6 +209,8 @@ class ProjectConfig:
         self.chromedriver: Optional[str] = chromedriver
         self.section_behavior: Optional[str] = section_behavior
 
+        self.statistics_generator: Optional[str] = statistics_generator
+
     @staticmethod
     def default_config(environment: SDocRuntimeEnvironment) -> "ProjectConfig":
         assert isinstance(environment, SDocRuntimeEnvironment)
@@ -224,6 +229,7 @@ class ProjectConfig:
             exclude_source_paths=[],
             test_report_root_dict={},
             source_nodes=[],
+            html2pdf_strict=False,
             html2pdf_template=None,
             bundle_document_version=ProjectConfig.DEFAULT_BUNDLE_DOCUMENT_VERSION,
             bundle_document_date=ProjectConfig.DEFAULT_BUNDLE_DOCUMENT_COMMIT_DATE,
@@ -235,6 +241,7 @@ class ProjectConfig:
             config_last_update=None,
             chromedriver=None,
             section_behavior=ProjectConfig.DEFAULT_SECTION_BEHAVIOR,
+            statistics_generator=None,
         )
 
     # Some server command settings can override the project config settings.
@@ -360,6 +367,9 @@ class ProjectConfig:
             ProjectFeature.TRACEABILITY_MATRIX_SCREEN in self.project_features
         )
 
+    def is_activated_tree_map(self) -> bool:
+        return ProjectFeature.TREE_MAP_SCREEN in self.project_features
+
     def is_activated_standalone_document(self) -> bool:
         return (
             ProjectFeature.STANDALONE_DOCUMENT_SCREEN in self.project_features
@@ -409,7 +419,14 @@ class ProjectConfig:
     def get_extra_static_files_path(self) -> str:
         return self.environment.get_extra_static_files_path()
 
+    def get_project_hash(self) -> str:
+        assert self.input_paths is not None and len(self.input_paths) > 0
+        return get_md5(self.input_paths[0])
+
     def shall_parse_nodes(self, path_to_file: str) -> bool:
+        if self.source_root_path is None:
+            return False
+
         for sdoc_source_config_entry_ in self.source_nodes:
             # FIXME: Move the setting of full paths to .finalize() of this config
             #        class when it is implemented.
@@ -421,6 +438,7 @@ class ProjectConfig:
             )
             if path_to_file.startswith(full_path):
                 return True
+
         return False
 
 
@@ -494,12 +512,13 @@ class ProjectConfigLoader:
         exclude_source_paths: List[str] = []
         test_report_root_dict: Dict[str, str] = {}
         source_nodes: List[Dict[str, str]] = []
+        html2pdf_strict: bool = False
         html2pdf_template: Optional[str] = None
         bundle_document_version = ProjectConfig.DEFAULT_BUNDLE_DOCUMENT_VERSION
         bundle_document_date = ProjectConfig.DEFAULT_BUNDLE_DOCUMENT_COMMIT_DATE
 
         traceability_matrix_relation_columns: Optional[
-            List[Tuple[str, str]]
+            List[Tuple[str, Optional[str]]]
         ] = None
         reqif_profile = ReqIFProfile.P01_SDOC
         reqif_multiline_is_xhtml = False
@@ -508,6 +527,7 @@ class ProjectConfigLoader:
         chromedriver: Optional[str] = None
 
         section_behavior: str = ProjectConfig.DEFAULT_SECTION_BEHAVIOR
+        statistics_generator: Optional[str] = None
 
         if "project" in config_dict:
             project_content = config_dict["project"]
@@ -537,6 +557,10 @@ class ProjectConfigLoader:
             if ProjectFeature.ALL_FEATURES in project_features:
                 project_features = ProjectFeature.all()
 
+            statistics_generator = project_content.get(
+                "statistics_generator", statistics_generator
+            )
+
             include_doc_paths = project_content.get(
                 "include_doc_paths", include_doc_paths
             )
@@ -610,6 +634,16 @@ class ProjectConfigLoader:
                         f"{exception_} Provided string: '{exclude_source_path}'."
                     ) from exception_
 
+            html2pdf_strict = project_content.get(
+                "html2pdf_strict", html2pdf_strict
+            )
+            if html2pdf_strict is not None:
+                if not isinstance(html2pdf_strict, bool):
+                    raise ValueError(
+                        "strictdoc.toml: 'html2pdf_strict': "
+                        f"must be a true/false value: {html2pdf_strict}."
+                    )
+
             html2pdf_template = project_content.get(
                 "html2pdf_template", html2pdf_template
             )
@@ -741,6 +775,7 @@ class ProjectConfigLoader:
             exclude_source_paths=exclude_source_paths,
             test_report_root_dict=test_report_root_dict,
             source_nodes=source_nodes,
+            html2pdf_strict=html2pdf_strict,
             html2pdf_template=html2pdf_template,
             bundle_document_version=bundle_document_version,
             bundle_document_date=bundle_document_date,
@@ -752,4 +787,5 @@ class ProjectConfigLoader:
             config_last_update=config_last_update,
             chromedriver=chromedriver,
             section_behavior=section_behavior,
+            statistics_generator=statistics_generator,
         )
