@@ -2,7 +2,6 @@
 @relation(SDOC-SRS-28, scope=file)
 """
 
-# mypy: disable-error-code="no-untyped-call,union-attr"
 import datetime
 from copy import deepcopy
 from typing import Any, Dict, Generator, List, Optional, Tuple, Union
@@ -14,8 +13,8 @@ from strictdoc.backend.sdoc.models.document_config import DocumentConfig
 from strictdoc.backend.sdoc.models.inline_link import InlineLink
 from strictdoc.backend.sdoc.models.node import SDocNode
 from strictdoc.backend.sdoc.models.section import SDocSection
-from strictdoc.backend.sdoc_source_code.models.range_marker import RangeMarker
 from strictdoc.backend.sdoc_source_code.models.source_file_info import (
+    RelationMarkerType,
     SourceFileTraceabilityInfo,
 )
 from strictdoc.core.asset_manager import AssetManager
@@ -44,6 +43,7 @@ from strictdoc.helpers.sorting import alphanumeric_sort
 class TraceabilityIndex:
     def __init__(
         self,
+        document_tree: DocumentTree,
         document_iterators: Dict[SDocDocument, DocumentCachingIterator],
         file_traceability_index: FileTraceabilityIndex,
         graph_database: GraphDatabase,
@@ -57,7 +57,7 @@ class TraceabilityIndex:
         )
 
         self.graph_database: GraphDatabase = graph_database
-        self.document_tree: Optional[DocumentTree] = None
+        self.document_tree: DocumentTree = document_tree
         self.asset_manager: Optional[AssetManager] = None
         self.file_dependency_manager: FileDependencyManager = (
             file_dependency_manager
@@ -68,6 +68,15 @@ class TraceabilityIndex:
             datetime.datetime.fromtimestamp(0)
         )
 
+        # The timestamp is used by HTML/JS for invalidating the search index
+        # cache in the IndexedDB database.
+        # If no documents have to be re-generated with the second+ run of
+        # StrictDoc, this timestamp is set of a modification date of the first
+        # SDoc document, see export_static_html_search_index.
+        self.search_index_timestamp: datetime.datetime = datetime.datetime.now(
+            datetime.timezone.utc
+        )
+
     @property
     def document_iterators(self) -> Dict[SDocDocument, DocumentCachingIterator]:
         return self._document_iterators
@@ -256,7 +265,7 @@ class TraceabilityIndex:
 
     def get_requirement_file_links(
         self, requirement: SDocNode
-    ) -> List[Tuple[str, List[RangeMarker]]]:
+    ) -> List[Tuple[str, List[RelationMarkerType]]]:
         return self._file_traceability_index.get_requirement_file_links(
             requirement
         )
@@ -527,10 +536,12 @@ class TraceabilityIndex:
         )
 
         # Mark document and parent document (if different) for re-generation.
+        assert document.meta is not None
         set_file_modification_time(
             document.meta.input_doc_full_path, datetime.datetime.today()
         )
         if parent_requirement_document != document:
+            assert parent_requirement_document.meta is not None
             set_file_modification_time(
                 parent_requirement_document.meta.input_doc_full_path,
                 datetime.datetime.today(),
@@ -598,10 +609,12 @@ class TraceabilityIndex:
         )
 
         # Mark document and parent document (if different) for re-generation.
+        assert document.meta is not None
         set_file_modification_time(
             document.meta.input_doc_full_path, datetime.datetime.today()
         )
         if child_requirement_document != document:
+            assert child_requirement_document.meta is not None
             set_file_modification_time(
                 child_requirement_document.meta.input_doc_full_path,
                 datetime.datetime.today(),
@@ -772,10 +785,12 @@ class TraceabilityIndex:
             )
 
         # Mark document and parent document (if different) for re-generation.
+        assert document.meta is not None
         set_file_modification_time(
             document.meta.input_doc_full_path, datetime.datetime.today()
         )
         if parent_requirement_document != document:
+            assert parent_requirement_document.meta is not None
             set_file_modification_time(
                 parent_requirement_document.meta.input_doc_full_path,
                 datetime.datetime.today(),
@@ -820,10 +835,12 @@ class TraceabilityIndex:
             )
 
         # Mark document and parent document (if different) for re-generation.
+        assert document.meta is not None
         set_file_modification_time(
             document.meta.input_doc_full_path, datetime.datetime.today()
         )
         if child_requirement_document != document:
+            assert child_requirement_document.meta is not None
             set_file_modification_time(
                 child_requirement_document.meta.input_doc_full_path,
                 datetime.datetime.today(),
@@ -1107,6 +1124,7 @@ class TraceabilityIndex:
             if document_.document_is_included():
                 continue
 
+            assert document_.ng_including_document_reference is not None
             document_.ng_including_document_reference.set_document(
                 bundle_document
             )
