@@ -2,7 +2,6 @@
 @relation(SDOC-SRS-28, SDOC-SRS-2, scope=file)
 """
 
-# mypy: disable-error-code="arg-type,no-untyped-call,union-attr"
 import datetime
 import glob
 import os
@@ -85,11 +84,13 @@ class TraceabilityIndexBuilder:
         latest_strictdoc_own_file = (
             max(strict_own_files, key=os.path.getctime)
             if len(strict_own_files) > 0
-            else 0
+            else None
         )
 
-        strictdoc_last_update: datetime.datetime = get_file_modification_time(
-            latest_strictdoc_own_file
+        strictdoc_last_update: datetime.datetime = (
+            get_file_modification_time(latest_strictdoc_own_file)
+            if (latest_strictdoc_own_file is not None)
+            else datetime.datetime.fromtimestamp(0)
         )
         if (
             project_config.config_last_update is not None
@@ -114,7 +115,6 @@ class TraceabilityIndexBuilder:
                 project_config,
             )
         )
-        traceability_index.document_tree = document_tree
         traceability_index.asset_manager = asset_manager
         traceability_index.strictdoc_last_update = strictdoc_last_update
 
@@ -183,15 +183,19 @@ class TraceabilityIndexBuilder:
                         node_document = assert_cast(
                             node_.get_document(), SDocDocument
                         )
+                        assert node_document.meta is not None
+
                         traceability_index.file_dependency_manager.add_dependency(
                             source_file.full_path,
                             source_file.output_file_full_path,
-                            node_document.meta.input_doc_full_path,
+                        )
+                        traceability_index.file_dependency_manager.add_dependency(
+                            source_file.full_path,
+                            node_document.meta.output_document_full_path,
                         )
                         traceability_index.file_dependency_manager.add_dependency(
                             node_document.meta.input_doc_full_path,
-                            node_document.meta.output_document_full_path,
-                            source_file.full_path,
+                            source_file.output_file_full_path,
                         )
 
             traceability_index.document_tree.attach_source_tree(source_tree)
@@ -269,6 +273,7 @@ class TraceabilityIndexBuilder:
         )
 
         traceability_index = TraceabilityIndex(
+            document_tree,
             d_01_document_iterators,
             file_traceability_index=d_07_file_traceability_index,
             graph_database=graph_database,
@@ -312,6 +317,14 @@ class TraceabilityIndexBuilder:
 
         document: SDocDocument
         for document in document_tree.document_list:
+            assert document.grammar is not None
+            assert document.meta is not None
+
+            traceability_index.file_dependency_manager.add_dependency(
+                document.meta.input_doc_full_path,
+                document.meta.output_document_full_path,
+            )
+
             if document.config.view_style_tag == "REQUIREMENT_STYLE":
                 DEPRECATION_ENGINE.add_message(
                     "DEPRECATED_REQUIREMENT_STYLE",
@@ -337,10 +350,27 @@ class TraceabilityIndexBuilder:
                         "TraceabilityIndex: "
                         f'the document "{document.reserved_title}" '
                         "imports a grammar from a file that does not exist: "
-                        f'"{document.grammar.import_from_file}".'
+                        f'"{document.grammar.import_from_file}". One known '
+                        f"source of this error is when only a single document "
+                        f"file is provided as input to the export or server "
+                        f"command, rather than the containing folder. To locate "
+                        f"the grammar file, StrictDoc needs to be able to "
+                        f"resolve it relative to the input path."
                     )
+
                 document.grammar.update_with_elements(document_grammar.elements)
 
+                # This is for the backward compatibility with the existing users.
+                # If the included project grammar has no TEXT element defined,
+                # we add it here automatically.
+                if not document.grammar.has_text_element():
+                    document.grammar.add_element_first(
+                        DocumentGrammar.create_default_text_element(
+                            document.grammar,
+                            enable_mid=document.config.enable_mid is True,
+                        )
+                    )
+
             # This is important because due to the difference between the
             # normal grammar vs imported grammar, the parent may not be set at
             # this point.
@@ -416,6 +446,7 @@ class TraceabilityIndexBuilder:
 
                 if isinstance(node, SDocNode):
                     try:
+                        assert document.grammar is not None
                         SDocValidator.validate_node(
                             node,
                             document_grammar=document.grammar,
@@ -489,7 +520,7 @@ class TraceabilityIndexBuilder:
                         rhs_node=node,
                     )
 
-                if node.is_requirement():
+                if isinstance(node, SDocNode):
                     requirement_node: SDocNode = assert_cast(node, SDocNode)
                     if requirement_node.reserved_tags is not None:
                         for tag in requirement_node.reserved_tags:
@@ -519,12 +550,14 @@ class TraceabilityIndexBuilder:
         requirement: SDocNode
 
         for document in document_tree.document_list:
+            assert document.meta is not None
+
             document_iterator = d_01_document_iterators[document]
 
             for node, _ in document_iterator.all_content(
                 print_fragments=False,
             ):
-                if not node.is_requirement():
+                if not isinstance(node, SDocNode):
                     continue
 
                 requirement = assert_cast(node, SDocNode)
@@ -534,7 +567,7 @@ class TraceabilityIndexBuilder:
                 # all UIDs or ANCHORS (they also have UIDs) are registered at the
                 # previous pass.
                 #
-                for node_field_ in node.enumerate_fields():
+                for node_field_ in requirement.enumerate_fields():
                     for part in node_field_.parts:
                         if isinstance(part, InlineLink):
                             if not graph_database.has_link(
@@ -593,18 +626,18 @@ class TraceabilityIndexBuilder:
                             parent_requirement.get_document(), SDocDocument
                         )
                         if document != parent_document:
+                            assert parent_document.meta is not None
+
                             # This is where we help the incremental generation to
                             # understand that the related documents must be
                             # re-generated together.
                             file_dependency_manager.add_dependency(
                                 document.meta.input_doc_full_path,
-                                document.meta.output_document_full_path,
-                                parent_document.meta.input_doc_full_path,
+                                parent_document.meta.output_document_full_path,
                             )
                             file_dependency_manager.add_dependency(
                                 parent_document.meta.input_doc_full_path,
-                                parent_document.meta.output_document_full_path,
-                                document.meta.input_doc_full_path,
+                                document.meta.output_document_full_path,
                             )
                     elif reference.ref_type == ReferenceType.CHILD:
                         child_reference: ChildReqReference = assert_cast(
@@ -639,18 +672,18 @@ class TraceabilityIndexBuilder:
                             child_requirement.get_document(), SDocDocument
                         )
                         if document != child_requirement_document:
+                            assert child_requirement_document.meta is not None
+
                             # This is where we help the incremental generation to
                             # understand that the related documents must be
                             # re-generated together.
                             file_dependency_manager.add_dependency(
                                 document.meta.input_doc_full_path,
-                                document.meta.output_document_full_path,
-                                child_requirement_document.meta.input_doc_full_path,
+                                child_requirement_document.meta.output_document_full_path,
                             )
                             file_dependency_manager.add_dependency(
                                 child_requirement_document.meta.input_doc_full_path,
-                                child_requirement_document.meta.output_document_full_path,
-                                document.meta.input_doc_full_path,
+                                document.meta.output_document_full_path,
                             )
                     else:
                         raise AssertionError(reference.ref_type)
@@ -665,10 +698,11 @@ class TraceabilityIndexBuilder:
             for node, _ in document_iterator.all_content(
                 print_fragments=False,
             ):
-                # FIXME: is_requirement() typing issue.
-                if not node.is_requirement():
+                if not isinstance(node, SDocNode):
                     continue
+
                 requirement = assert_cast(node, SDocNode)
+
                 if requirement.reserved_uid is None:
                     continue
 
@@ -721,6 +755,8 @@ class TraceabilityIndexBuilder:
 
         map_documents_by_input_rel_path: Dict[str, SDocDocument] = {}
         for document_ in document_tree.document_list:
+            assert document_.meta is not None
+
             map_documents_by_input_rel_path[
                 document_.meta.input_doc_full_path
             ] = document_
@@ -822,10 +858,9 @@ class TraceabilityIndexBuilder:
                         traceability_index.get_document_iterator(document)
                     )
                     for node, _ in document_iterator.all_content():
-                        if (
-                            node.is_section
-                            and not sections_query_object.evaluate(node)
-                        ):
+                        if isinstance(
+                            node, SDocSection
+                        ) and not sections_query_object.evaluate(node):
                             node.ng_whitelisted = False
                             # If the node is the last one, we check if all other
                             # nodes are filtered out and if so, mark the parent
@@ -839,10 +874,9 @@ class TraceabilityIndexBuilder:
                                 if isinstance(node.parent, SDocSection):
                                     node.parent.blacklist_if_needed()
 
-                        elif (
-                            node.is_requirement()
-                            and not requirements_query_object.evaluate(node)
-                        ):
+                        elif isinstance(
+                            node, SDocNode
+                        ) and not requirements_query_object.evaluate(node):
                             node.ng_whitelisted = False
                             # If the node is the last one, we check if all other
                             # nodes are filtered out and if so, mark the parent
@@ -853,8 +887,7 @@ class TraceabilityIndexBuilder:
                                 ]
                                 == node
                             ):
-                                if node.parent.is_section():
-                                    node.parent.blacklist_if_needed()
+                                node.parent.blacklist_if_needed()
 
             except (AttributeError, NameError, TypeError) as attribute_error_:
                 print(  # noqa: T201
