@@ -1,4 +1,3 @@
-# mypy: disable-error-code="arg-type,no-untyped-call,union-attr"
 from typing import List, Optional, Set, Tuple
 
 from jinja2 import Template
@@ -13,6 +12,7 @@ from strictdoc.backend.sdoc.models.grammar_element import (
     GrammarElement,
     GrammarElementField,
     GrammarElementFieldString,
+    GrammarElementFieldType,
     GrammarElementRelationChild,
     GrammarElementRelationFile,
     GrammarElementRelationParent,
@@ -80,6 +80,12 @@ class GrammarFormField:
     def get_input_field_human_title(self) -> str:
         return f"document_grammar_field[{self.field_mid}][field_human_title]"
 
+    def get_input_field_required(self) -> str:
+        return f"document_grammar_field[{self.field_mid}][field_required]"
+
+    def get_input_field_required_value(self) -> str:
+        return "true" if self.field_required else "false"
+
 
 @auto_described
 class GrammarFormRelation:
@@ -110,6 +116,9 @@ class GrammarElementFormObject(ErrorObject):
         document_mid: str,
         element_mid: str,
         element_name: str,
+        is_composite: Optional[bool],
+        prefix: Optional[str],
+        view_style: Optional[str],
         fields: List[GrammarFormField],
         relations: List[GrammarFormRelation],
         project_config: ProjectConfig,
@@ -120,6 +129,9 @@ class GrammarElementFormObject(ErrorObject):
         self.document_mid = document_mid
         self.element_mid: str = element_mid
         self.element_name: str = element_name
+        self.is_composite: Optional[bool] = is_composite
+        self.prefix: Optional[str] = prefix
+        self.view_style: Optional[str] = view_style
         self.fields: List[GrammarFormField] = fields
         self.relations: List[GrammarFormRelation] = relations
         self.project_config: ProjectConfig = project_config
@@ -133,6 +145,8 @@ class GrammarElementFormObject(ErrorObject):
         project_config: ProjectConfig,
         jinja_environment: JinjaEnvironment,
     ) -> "GrammarElementFormObject":
+        assert document.grammar is not None
+
         form_object_fields: List[GrammarFormField] = []
         form_object_relations: List[GrammarFormRelation] = []
         request_form_data_as_list = [
@@ -143,26 +157,35 @@ class GrammarElementFormObject(ErrorObject):
             parse_form_data(request_form_data_as_list), dict
         )
 
-        element_mid = request_form_dict["element_mid"]
+        element_mid = assert_cast(request_form_dict["element_mid"], str)
+
+        element_is_composite = request_form_dict["is_composite"]
+        element_prefix = request_form_dict["prefix"]
+        element_view_style = request_form_dict["view_style"]
 
         #
         # Grammar fields.
         #
-        document_grammar_fields = request_form_dict["document_grammar_field"]
+        document_grammar_fields = assert_cast(
+            request_form_dict["document_grammar_field"], dict
+        )
         for field_mid, field_dict in document_grammar_fields.items():
             assert isinstance(field_dict, dict), type(field_dict)
 
-            field_name = field_dict["field_name"]
+            field_name = assert_cast(field_dict["field_name"], str)
             field_human_title = field_dict.get("field_human_title")
             if field_human_title is not None:
                 field_human_title = field_human_title.strip()
                 if len(field_human_title) == 0:
                     field_human_title = None
+            field_required_string = field_dict.get("field_required")
+            field_required = field_required_string == "true"
+
             form_object_field = GrammarFormField(
                 field_mid=field_mid,
                 field_name=field_name,
                 field_human_title=field_human_title,
-                field_required=False,
+                field_required=field_required,
                 reserved=is_reserved_field(field_name),
             )
             form_object_fields.append(form_object_field)
@@ -170,14 +193,14 @@ class GrammarElementFormObject(ErrorObject):
         #
         # Grammar relations.
         #
-        document_grammar_relations = request_form_dict.get(
-            "document_grammar_relation", {}
+        document_grammar_relations = assert_cast(
+            request_form_dict.get("document_grammar_relation", {}), dict
         )
         for field_mid, field_dict in document_grammar_relations.items():
             assert isinstance(field_dict, dict), type(field_dict)
 
-            field_type = field_dict["type"]
-            field_role = field_dict["role"].strip()
+            field_type = assert_cast(field_dict["type"], str)
+            field_role = assert_cast(field_dict["role"].strip(), str)
 
             form_object_relation = GrammarFormRelation(
                 relation_mid=field_mid,
@@ -194,6 +217,15 @@ class GrammarElementFormObject(ErrorObject):
             document_mid=document.reserved_mid,
             element_mid=element_mid,
             element_name=element.tag,
+            is_composite=(
+                True
+                if element_is_composite == "true"
+                else False
+                if element_is_composite == "false"
+                else None
+            ),
+            prefix=assert_cast(element_prefix, str),
+            view_style=assert_cast(element_view_style, str),
             fields=form_object_fields,
             relations=form_object_relations,
             project_config=project_config,
@@ -236,6 +268,9 @@ class GrammarElementFormObject(ErrorObject):
             document_mid=document.reserved_mid,
             element_mid=element_mid,
             element_name=element.tag,
+            is_composite=element.property_is_composite,
+            prefix=element.property_prefix,
+            view_style=element.property_view_style,
             fields=grammar_form_fields,
             relations=grammar_form_relations,
             project_config=project_config,
@@ -270,7 +305,10 @@ class GrammarElementFormObject(ErrorObject):
             else:
                 fields_so_far.add(field.field_name)
 
-        if len(self.relations) == 0 and self.element_name != "TEXT":
+        if len(self.relations) == 0 and self.element_name not in (
+            "TEXT",
+            "SECTION",
+        ):
             self.add_error(
                 "Relations_Row",
                 (
@@ -322,7 +360,7 @@ class GrammarElementFormObject(ErrorObject):
     def convert_to_grammar_element(
         self, existing_grammar: DocumentGrammar
     ) -> GrammarElement:
-        grammar_fields: List[GrammarElementField] = []
+        grammar_fields: List[GrammarElementFieldType] = []
         for field in self.fields:
             grammar_field = GrammarElementFieldString(
                 parent=None,
@@ -362,13 +400,19 @@ class GrammarElementFormObject(ErrorObject):
                 raise NotImplementedError(relation)
 
         existing_element = existing_grammar.get_element_by_mid(self.element_mid)
+
         requirement_element = GrammarElement(
             parent=None,
             tag=existing_element.tag,
-            # FIXME: MERGE NODES.
-            property_is_composite="",
-            property_prefix="",
-            property_view_style="",
+            property_is_composite=(
+                "True"
+                if self.is_composite == True
+                else "False"
+                if self.is_composite == False
+                else ""
+            ),
+            property_prefix=self.prefix or "",
+            property_view_style=self.view_style or "",
             fields=grammar_fields,
             relations=relation_fields,
         )
@@ -462,3 +506,25 @@ class GrammarElementFormObject(ErrorObject):
             action="update",
             target="modal",
         )
+
+    def get_input_field_is_composite(self) -> str:
+        return "is_composite"
+
+    def get_input_field_is_composite_value(self) -> str:
+        if self.is_composite is not None:
+            if self.is_composite == True:
+                return "true"
+            return "false"
+        return ""
+
+    def get_input_field_prefix(self) -> str:
+        return "prefix"
+
+    def get_input_field_prefix_value(self) -> str:
+        return self.prefix or ""
+
+    def get_input_field_view_style(self) -> str:
+        return "view_style"
+
+    def get_input_field_view_style_value(self) -> str:
+        return self.view_style or ""
