@@ -1,3 +1,15 @@
+"""
+Model class that holds the default project metrics data.
+
+NOTE: If a user wants to customize the default project statistics screen by
+      implementing their own statistics generator, they will likely need to
+      create a custom version of this model class with their own set of metric
+      fields. This class serves as an example of how project metrics and
+      statistics can be structured and stored.
+
+@relation(SDOC-SRS-97, scope=file)
+"""
+
 from collections import defaultdict
 from dataclasses import dataclass, field
 from typing import Dict, List, Optional
@@ -16,8 +28,14 @@ class DocumentTreeStats:
     total_documents: int = 0
     total_requirements: int = 0
     total_sections: int = 0
+    total_source_files: int = 0
+    total_source_files_complete_coverage: int = 0
+    total_source_files_partial_coverage: int = 0
+    total_source_files_no_coverage: int = 0
+
     total_tbd: int = 0
     total_tbc: int = 0
+
     git_commit_hash: Optional[str] = None
 
     # Section.
