@@ -1,4 +1,3 @@
-# mypy: disable-error-code="arg-type,no-untyped-call,union-attr"
 import html
 from typing import Any, Dict, Optional, Tuple, Union
 
@@ -62,7 +61,7 @@ class LinkRenderer:
             # UID is unique enough.
             local_anchor = self._string_to_link(node.reserved_uid)
         else:
-            # If an element as no UID, provide some uniqueness.
+            # If an element has no UID, provide some uniqueness.
             unique_prefix = node.context.title_number_string
             if isinstance(node, (SDocSection, SDocDocument)):
                 local_anchor = (
@@ -78,8 +77,7 @@ class LinkRenderer:
                         f"{self._string_to_link(node.reserved_title)}"
                     )
                 else:
-                    # TODO: This is not reliable
-                    local_anchor = str(id(node))
+                    local_anchor = self._string_to_link(node.reserved_mid)
             else:
                 raise NotImplementedError(node)
         self.local_anchor_cache[node] = local_anchor
@@ -104,11 +102,12 @@ class LinkRenderer:
         ), node
 
         if isinstance(node, SDocDocument):
-            context_level_or_none = (
-                context_document.meta.level
-                if context_document is not None
-                else None
-            )
+            context_level_or_none: Optional[int] = None
+            if context_document is not None:
+                assert context_document.meta is not None
+                context_level_or_none = context_document.meta.level
+
+            assert node.meta is not None
             document_link = node.meta.get_html_link(
                 document_type,
                 context_level_or_none,
@@ -133,13 +132,15 @@ class LinkRenderer:
             return f"#{local_link}"
 
         # Now two cases:
-        # - Context document exists and we want to take into account this
+        # - Context document exists, and we want to take into account this
         # document's depth.
         # - Context document does not exist, such as when we are on a Search
         # screen. In this case, the level is simply zero.
-        level: int = (
-            context_document.meta.level if context_document is not None else 0
-        )  # FIXME 0 or 1?
+        level: int = 0
+        if context_document is not None:
+            assert context_document.meta is not None
+            level = context_document.meta.level
+
         link_cache_key = (document_type, level)
         if link_cache_key in self.req_link_cache:
             document_type_cache = self.req_link_cache[link_cache_key]
@@ -147,7 +148,10 @@ class LinkRenderer:
                 return document_type_cache[node]
         else:
             self.req_link_cache[link_cache_key] = {}
-        document_link = node.parent_or_including_document.meta.get_html_link(
+
+        parent_or_including_document = node.get_parent_or_including_document()
+        assert parent_or_including_document.meta is not None
+        document_link = parent_or_including_document.meta.get_html_link(
             document_type,
             level,
         )
@@ -167,10 +171,13 @@ class LinkRenderer:
         """
 
         assert isinstance(node, (SDocNode, SDocSection, Anchor)), node
+
+        parent_or_including_document = node.get_parent_or_including_document()
+        assert parent_or_including_document is not None
+        assert parent_or_including_document.meta is not None
+
         local_link = self.render_local_anchor(node)
-        document_link = (
-            node.parent_or_including_document.meta.get_html_doc_link()
-        )
+        document_link = parent_or_including_document.meta.get_html_doc_link()
         requirement_link = f"{document_link}#{local_link}"
         return requirement_link
 
@@ -189,6 +196,7 @@ class LinkRenderer:
             self.req_link_cache[link_cache_key] = {}
 
         document = assert_cast(node.get_document(), SDocDocument)
+        assert document.meta is not None
         document_link = document.meta.get_html_link(
             DocumentType.DOCUMENT, source_file.level + 1
         )
@@ -206,6 +214,7 @@ class LinkRenderer:
         document: SDocDocument = assert_cast(
             requirement.ng_document_reference.get_document(), SDocDocument
         )
+        assert document.meta is not None
         path_prefix = document.meta.get_root_path_prefix()
         source_file_link = (
             f"{path_prefix}/_source_files/{requirement_source_path}.html"
@@ -250,7 +259,7 @@ class LinkRenderer:
 
     @staticmethod
     def render_requirement_in_source_file_range_link_using_id(
-        req_uid: str,  # noqa: ARG004
+        req_uid: Optional[str],  # noqa: ARG004
         source_link: str,
         context_source_file: SourceFile,
         source_range: RangeMarker,
@@ -259,6 +268,9 @@ class LinkRenderer:
         assert isinstance(context_source_file, SourceFile)
         assert len(source_link) > 0
 
+        if req_uid is None:
+            req_uid = ""
+
         def get_root_path_prefix(level: int) -> str:
             assert level > 0
             return ("../" * level)[:-1]
