@@ -1,5 +1,4 @@
 # pragma: no cover file.
-# mypy: disable-error-code="arg-type,no-untyped-call,union-attr"
 
 """
 Experimental code to generate SPDX.
@@ -60,6 +59,7 @@ from strictdoc.core.traceability_index import TraceabilityIndex
 from strictdoc.export.spdx.spdx_sdoc_container import SPDXSDocContainer
 from strictdoc.export.spdx.spdx_to_sdoc_converter import SPDXToSDocConverter
 from strictdoc.helpers.cast import assert_cast
+from strictdoc.helpers.file_system import file_open_read_bytes
 from strictdoc.helpers.sha256 import get_sha256
 
 RELATION_ID_HOW_TO = "SPDXRef-Relationship-How-to-form-ID?"
@@ -80,8 +80,12 @@ def get_spdx_ref(node: Union[SDocDocument, SDocNode, str]) -> str:
         identifier = node.reserved_mid
     elif node.reserved_uid is not None:
         identifier = node.reserved_uid
-    else:
+    elif node.reserved_title is not None:
         identifier = re.sub(r"[ #-/\\]", "_", node.reserved_title)
+    else:
+        raise RuntimeError(
+            f"Cannot create unique identifier for this node: {node}"
+        )
 
     if isinstance(node, SDocDocument):
         return "SDocDocument-" + identifier
@@ -137,6 +141,8 @@ class SDocToSPDXConverter:
     def create_document_to_file(
         document: SDocDocument, document_bytes: bytes
     ) -> File:
+        assert document.meta is not None
+
         return File(
             spdx_id=f"SPDXRef-File-{get_spdx_ref(document)}",
             name=document.meta.document_filename,
@@ -177,6 +183,8 @@ class SDocToSPDXConverter:
             document_bytes[requirement.ng_byte_start : requirement.ng_byte_end]
         )
         assert requirement.reserved_uid is not None
+        assert requirement.reserved_statement is not None
+
         return Snippet(
             spdx_id=requirement.reserved_uid,
             primary_purpose=SoftwarePurpose.DOCUMENTATION,
@@ -246,7 +254,10 @@ class SPDXGenerator:
         lookup_file_name_to_spdx_file: Dict[str, File] = {}
 
         for document_ in traceability_index.document_tree.document_list:
-            with open(document_.meta.input_doc_full_path, "rb") as input_file_:
+            assert document_.meta is not None
+            with file_open_read_bytes(
+                document_.meta.input_doc_full_path
+            ) as input_file_:
                 document_bytes = input_file_.read()
 
             #
@@ -278,7 +289,10 @@ class SPDXGenerator:
             )
 
             for node, _ in document_iterator.all_content():
-                if node.is_requirement():
+                if (
+                    isinstance(node, SDocNode)
+                    and node.node_type == "REQUIREMENT"
+                ):
                     if node.reserved_uid is None:
                         continue
 
@@ -329,7 +343,9 @@ class SPDXGenerator:
                         if node_link_path_ in lookup_file_name_to_spdx_file:
                             continue
 
-                        with open(node_link_path_, "rb") as node_link_file_:
+                        with file_open_read_bytes(
+                            node_link_path_
+                        ) as node_link_file_:
                             file_bytes = node_link_file_.read()
 
                         source_spdx_file = (
@@ -370,6 +386,9 @@ class SPDXGenerator:
                 document_
             )
             for node, _ in document_iterator.all_content():
+                if not isinstance(node, SDocNode):
+                    continue
+
                 if node.is_requirement():
                     if node.reserved_uid is None:
                         continue
